
# q-expansion of newform 144.12.a.j, downloaded from the LMFDB on 05 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 144
weight = 12
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [3490], [55464], [-597004], [1373878], [-10140850], [7297396], [-32057464], [13605402], [-233160800], [-257786178], [221438598], [1697758892], [527509392], [-3277379822], [-3001908988], [-11630023610], [17189000548], [26169539608], [-7039021094], [4199910416], [-39739936436], [-10565331594], [-69851645662], [-74519412606], [-78664150648], [78848972820], [-70868029994], [-19681604562], [190251579008], [114524784988], [-117101162394], [-989560526196], [797579218418], [-1183229001992], [-753512032922], [-834305483964], [-1015135845576], [-2894963106966], [-4092633678996], [508615079022], [2999513064000], [-2181925750462], [4774752410466], [-295210163608], [-4768855897804], [9214548039008], [-5854438063236], [-14027335387394], [5296101013318], [-13164628430384], [22589663991378], [26118547450180], [12657981060350], [5370231542360], [8601398737994], [6013709361616], [-26331197092466], [51174845289782], [-8948955462308], [26847972185154], [-6624331875564], [40285233723624], [-21542297569494], [2794787240634], [44856017296972], [-52342753988238], [162474635223396], [56070596402598], [-121433098473250], [99308194169208], [-58096458156880], [10633630708910], [194563767957820], [-219102152231808], [-258688254353630], [-253970045277898], [-176194300155186], [254652202822986], [-10976383980612], [-306791483729602], [-194136614783984], [-197020802576750], [481833288711576], [-472499076448892], [35241370012606], [541522131945818], [263903025105418], [34496762783632], [-588837802998324], [-173689784193632], [-842617287500536], [-538407736241004], [136233889206996], [488252435324712], [488013674701434], [-1107897067126810], [618140078509580], [761294937204198], [565709123299012], [1880894854259434], [-176230678421524], [491855357914582], [-918553298629380], [-1538421618029374], [5660273583284], [411514325733262], [3353751804185160], [1025019084513418], [1990791782045920], [-516975009730690], [678132802269894], [655793062849004], [-336387833915560], [2680164396379006], [-163908120381276], [-3113190842401064], [-3156735277314102], [-2219965096229364], [2945325261081614], [2039719577287138], [-147928819164734], [-4940163148028716], [-2964030774710636], [-845017864698822], [-6161944565636962], [7674671417903984], [-2553817398932424], [10663897197233702], [-4939672509585788], [10483772357498360], [9786966289594416], [-493316791880914], [-8971679370118762], [3726504971788802], [-1123373503759710], [13968284524139636], [-17722640561243366], [13527097038061958], [-11163406521710164], [12963905005939218], [3202381470941464], [9928785760151044], [11793494274289990], [-290602147861224], [-15798108662290354], [-16482619724756106], [62183817984156], [-17713423825522144], [8215556356233942], [19185753602969390], [-10441168811139500], [9339120463680424], [13597564581061004], [24491838693488048], [-19471493195311624], [37246532785572638], [-3425214069982470], [26922797018900586], [-28732237659322260], [17745400909311318], [-24560234585384600], [-4691582779295436], [27787204933173774], [-10951940338723128], [-33627950376561568], [52710991601358334], [-30695997568522414], [-5153178272522798], [30761909213005380], [17472137176074630], [11322026119404376], [18417425685096730], [57662128620619984], [-23291500707834826], [20590902958954588], [-17615531847767742], [28599839564372552], [-77477004781865194], [2436053098494784], [71600785359442588], [65851681835483934], [-28360149744158682], [-76102936538888208], [-65556044601338190], [-4125638778482522], [36116838272505348], [73668889145401402], [-51118686401577600], [-62866591808576126], [69229438240358772], [-23288554403387404], [-107134563963460454], [126890083999449276], [105836757822975494], [107239031288113810], [92702306740006342], [45055359521082302], [-103421323739946600], [123827694238321418], [-85447044101173616], [-113803496580034354], [180597067009757602], [-69237904241604204], [-157655829343673478], [-134403738460581632], [153605836462599900], [-72437803160029210], [134812931446088972], [21477117524247474], [183770318696316018], [186194537721598328], [160935590688415396], [-73690655547500360], [-281817687316675462], [55024011355824496], [-26092683432859250], [103135444421149512], [28807275768414810], [-34667759207848834], [189147904681406040], [71739436824943230], [186317454806408272], [-325625992093832436], [270009878043399950], [-151815243410028618], [54754673649264864], [59531895089821128], [-274254209750852140], [-174819641319650922], [-334494312722232940], [-185051391967304000], [-143771476371341402], [-301628924402036532], [386205427496223856], [218368551674113010], [120225467478586930], [-242197667442241820], [375968159047540472], [-169280080184334036], [-28492334140056644], [137591730669125288], [-610734915636775754], [73682786401079886], [470520738363429256], [141855109473389984], [-115395659922223812], [21142979042157740], [-636583740038725232], [378196824989136966], [-282934173991192770], [-515908226409112040], [100873848151011802], [149530473318248586], [-513731115567559860], [-193349908177017010], [260140069947924380], [-557288318176394622], [-311499150857495318], [966522177564195968], [-661814643309752484], [-451379026085782562], [-754456362646702746], [79220015793005086], [-802054425138086588], [241147280904550250], [-808636056355228330], [588654711629952892], [-1063809477199831198], [714564085175411446], [-572622206555909324], [-1086660831161903606], [515765060591360864], [715066263137474386], [725320931132461528], [513456832749596484], [-428695498950779770], [-1550506713856360934], [-1530610405621824628], [1068658250785482080], [-1217094320929261240], [408977977117860584], [-211668730943405026], [489849372711512140], [530955764556787952], [915840783328117874], [-814232969399147598], [-586004072121721160], [1501274699951349470], [399710084902576426], [-670892551354378836], [-372354376775864042], [-1208748981697444236], [1065667712541626166], [-969811885613344358], [626689003338004768], [849698385730808466], [-2384034475853158524], [-934545138288123164], [-1821642284847701158], [-1982534223419274742], [-950729585655714416], [1761199221757012940], [-1352732403601413348], [-1561731337666747486], [477476173076189844], [2048939155976944214], [-2591142256429852376], [-634963049031707554], [-2501731368398449872], [-2664829232088845454], [-3274040040904296034], [160693127754173636], [578266169742225848], [590169867221197946], [-415288678197420564], [55207192929853632], [2734932872266861250], [472025606344895886], [2500246180157666996], [-3003219305022235766], [3039579662779399514], [-187660942949767456], [1261827343280958406], [377667663492845778], [1551319622796438180], [2501882816274598620], [367306253487258080], [-931180130224708158], [3185779444654772374], [3649000406549974074], [-3771720569886659136], [-1784471492182245348], [-549871214125555764], [-3939885815791600668], [4016712094672535590], [1895320143553723998], [20157502720093370], [1774121621020933680], [-3042973829630242514], [2644003382168958870], [-1055193736749133662], [-4321031352329534040], [2820996937594559770], [1892139756408048188], [2006943040494833598], [1110836478582450092], [6813207498958293648], [-6103259209037926894], [-1696019095576531356], [3613655419108805594], [6778587487361566090], [6276221285979062544], [1080437358559950926], [-2703848961709824394], [-4246968705122436064], [743388192337784596], [5829329171705158702], [3160632522951886248], [-1663665425361777442], [-6093394495198892442], [-7072607073534527568], [5631721462331493412], [4200785593809838148], [-1509532848076551942], [-8695116630283230102], [6935711411276246888], [-61706811343580150], [-708173394720769412], [2350928856997992300], [-1019866821825186608], [3800937671944716754], [9415566783507716312], [-171947239680143482], [145225218889713804], [-1065495462231777696], [-6116033277147964574], [59129121161303022], [6360072294436577834], [-2268933923428905030], [-9364739102325182170], [-4906640979567819336], [-3972337365038792226], [-8458058529150841852], [3582748796513074296], [-1104010187614586448], [-3380097520337937490], [2479530694091959868], [-12443246796424659072], [1482907744667170690], [-4775637852037419998], [8868131694916161396], [13805123027521282548], [-11225384882925525240], [-8338884805924310966], [3565735290615924768], [2937713961286033158], [-8153028608977124564], [10157213608002831250], [7947369872851994054], [8008850520968960190], [-3526408035092300656], [6427141393434700022], [-11674750324281108734], [-2508957452707653112], [-1834532119126296746], [-12507398988638108754], [-11677214487021932588], [-426198227293059876], [988654410315769010], [-7973563892242177934], [-16504849835147078492], [-9608569791567554620], [-12153505175973150342], [-2107152415671420950], [-1037308255210595904], [19424174707948589800], [9676598213993711502], [19242042854605698376], [-14517848034499733926], [-19468321629086678914], [2663251506217290064], [-18878440789524777788], [5731144671310270618], [11997370716074805962], [10674111333502981388], [8477266378764027318], [-11213219845118541348], [14859729491041394792]]
